from django.contrib.auth.backends import ModelBackend
from django.contrib.auth.models import User
from django.db import models
from django.db.models.signals import post_save

from python_digest import calculate_partial_digest

from django_digest.utils import get_backend, get_setting, DEFAULT_REALM

class UserNonce(models.Model):
    user = models.ForeignKey(User)
    nonce = models.CharField(max_length=100, unique=True, db_index=True)
    count = models.IntegerField(null=True)
    last_used_at = models.DateTimeField(null=False)
    class Meta:
        ordering = ('last_used_at',)

class PartialDigest(models.Model):
    user = models.ForeignKey(User)
    login = models.CharField(max_length=75, db_index=True)
    partial_digest = models.CharField(max_length=100)
    confirmed = models.BooleanField(default=True)
    
_postponed_partial_digests = {}

def _get_logins(user, method_name):
    login_factory = get_backend('DIGEST_LOGIN_FACTORY',
                                'django_digest.DefaultLoginFactory')
    method = getattr(login_factory, method_name, None)
    if method:
        return set(method(user))
    else:
        return set()

def _confirmed_logins(user):
    return _get_logins(user, 'confirmed_logins_for_user')

def _unconfirmed_logins(user):
    return _get_logins(user, 'unconfirmed_logins_for_user')
 
def _store_partial_digests(user):
    PartialDigest.objects.filter(user=user).delete()
    for (login, partial_digest, confirmed) in (
        _postponed_partial_digests[user.password]):
        PartialDigest.objects.create(user=user, login=login, confirmed=confirmed,
                                     partial_digest=partial_digest)

def _prepare_partial_digests(user, raw_password):
    if raw_password is None:
        return
    realm = get_setting('DIGEST_REALM', DEFAULT_REALM)
    partial_digests = []
    for (confirmed, factory_method) in ((True, _confirmed_logins),
                                        (False, _unconfirmed_logins)):
        partial_digests += [(login, calculate_partial_digest(login, realm,
                                                             raw_password), confirmed)
                            for login in factory_method(user)]

    password_hash = user.password
    _postponed_partial_digests[password_hash] = partial_digests

_old_set_password = User.set_password
_old_check_password = User.check_password
_old_authenticate = ModelBackend.authenticate

def _review_partial_digests(user):
    confirmed_logins = _confirmed_logins(user)
    unconfirmed_logins = _unconfirmed_logins(user)

    for pd in PartialDigest.objects.filter(user=user):
        if pd.login in confirmed_logins:
            if not pd.confirmed:
                pd.confirmed = True
                pd.save()
        elif pd.login in unconfirmed_logins:
            if pd.confirmed:
                pd.confirmed = False
                pd.save()
        else:
            pd.delete()
        
    
def _after_authenticate(user, password):
    for (confirmed, factory_method) in ((True, _confirmed_logins),
                                        (False, _unconfirmed_logins)):
        logins = factory_method(user)
        # if we don't have all of these logins
        # and exactly these logins in the database
        db_logins = set(
            [pd.login for pd in PartialDigest.objects.filter(user=user,
                                                             confirmed=confirmed)])
        if db_logins != logins:
            _prepare_partial_digests(user, password)
            _persist_partial_digests(user)
            return

def _new_check_password(user, raw_password):
    result = _old_check_password(user, raw_password)
    if result:
        _after_authenticate(user, raw_password)
    return result

def _new_authenticate(backend, username=None, password=None):
    user = _old_authenticate(backend, username, password)
    if user:
        _after_authenticate(user, password)
    return user
        
def _new_set_password(user, raw_password):
    _old_set_password(user, raw_password)
    _prepare_partial_digests(user, raw_password)

User.check_password = _new_check_password    
User.set_password = _new_set_password
ModelBackend.authenticate = _new_authenticate

def _persist_partial_digests(user):
    password_hash = user.password
    if password_hash in _postponed_partial_digests:
        _store_partial_digests(user)
        del _postponed_partial_digests[password_hash]

def _post_save_persist_partial_digests(sender, instance=None, **kwargs):
    if instance is not None:
        _persist_partial_digests(instance)

post_save.connect(_post_save_persist_partial_digests, sender=User)



import logging

# Make sure a NullHandler is available
# This was added in Python 2.7/3.2
try:
    from logging import NullHandler
except ImportError:
    class NullHandler(logging.Handler):
        def emit(self, record):
            pass

_l = logging.getLogger(__name__)
_l.addHandler(NullHandler())
_l.setLevel(logging.DEBUG)

from copy import copy

from django.conf import settings
from django.core import signals
from django import db
from django.db.utils import DEFAULT_DB_ALIAS

# Backwards compatibility
from django_digest.models import (
    _after_authenticate as update_partial_digests,
    _review_partial_digests as review_partial_digests
)
from django_digest.backend.storage import AccountStorage, NonceStorage


class MultiDb(object):
    def __init__(self, using=DEFAULT_DB_ALIAS, create=False):
        self.created = create and not self.is_test_mode()
        if self.created:
            self.connection = self.create_connection(using=using)
        else:
            self.connection = db.connections[using]
        self.init_signals()

    def init_signals(self):
        signals.request_finished.connect(self.close_connection)
        signals.request_started.connect(self.reset_queries)

    def create_connection(self, using=None, **kwargs):
        if using:
            params = copy(db.connections[using].settings_dict)
            params.update(kwargs)
        else:
            params = kwargs
        return db.backend.DatabaseWrapper(params)

    def close_connection(self, **kwargs):
        if self.created and self.connection:
            self.connection.close()

    def reset_queries(self, **kwargs):
        if self.created and self.connection:
            self.connection.queries = []

    def is_test_mode(self):
        # hack - in tests we want to run within the same transaction
        # as the test case to pick up users created there
        from django.core import mail
        return hasattr(mail, 'outbox')

    def commit(self):
        if self.created and self.connection and not self.is_test_mode():
            self.connection._commit()


class FakeMultiDb(MultiDb):
    """For Django < 1.2."""
    def __init__(self):
        self.created = not self.is_test_mode()
        self.connection = self.create_connection()
        self.init_signals()

    def create_connection(self, **kwargs):
        if self.is_test_mode():
            return db.connection
        params = {'DATABASE_HOST': settings.DATABASE_HOST,
                  'DATABASE_NAME': settings.DATABASE_NAME,
                  'DATABASE_OPTIONS': settings.DATABASE_OPTIONS,
                  'DATABASE_PASSWORD': settings.DATABASE_PASSWORD,
                  'DATABASE_PORT': settings.DATABASE_PORT,
                  'DATABASE_USER': settings.DATABASE_USER,
                  'TIME_ZONE': settings.TIME_ZONE}
        params.update(kwargs)
        return db.backend.DatabaseWrapper(params)



from __future__ import with_statement
from django.test import TestCase, TransactionTestCase

from contextlib import contextmanager
import time

from mocker import Mocker, expect

import python_digest
from python_digest.utils import parse_parts

from django.conf import settings
from django.contrib.auth.models import User
from django.db import transaction
from django.http import HttpRequest
from django.utils.functional import LazyObject

from django_digest import HttpDigestAuthenticator
from django_digest.backend.db import AccountStorage
from django_digest.decorators import httpdigest
from django_digest.middleware import HttpDigestMiddleware
from django_digest.models import PartialDigest
from django_digest.utils import get_setting, get_backend, DEFAULT_REALM

@contextmanager
def patch(namespace, **values):
    """Patches `namespace`.`name` with `value` for (name, value) in values"""
    
    originals = {}

    if isinstance(namespace, LazyObject):
        if namespace._wrapped is None:
            namespace._setup()
        namespace = namespace._wrapped

    for (name, value) in values.iteritems():
        try:
            originals[name] = getattr(namespace, name)
        except AttributeError:
            originals[name] = NotImplemented
        if value is NotImplemented:
            if originals[name] is not NotImplemented:
                delattr(namespace, name)
        else:
            setattr(namespace, name, value)

    try:
        yield
    finally:
        for (name, original_value) in originals.iteritems():
            if original_value is NotImplemented:
                if values[name] is not NotImplemented:
                    delattr(namespace, name)
            else:
                setattr(namespace, name, original_value) 

class DummyBackendClass(object):
    pass

class OtherDummyBackendClass(object):
    pass

DUMMY_SETTINGS = {
    'SECRET_KEY': 'sekret',
    'DIGEST_ENFORCE_NONCE_COUNT': NotImplemented,
    'DIGEST_REALM': NotImplemented,
    'DIGEST_NONCE_TIMEOUT_IN_SECONDS': NotImplemented,
    'DIGEST_REQUIRE_AUTHENTICATION': NotImplemented,
    'DIGEST_LOGIN_FACTORY': NotImplemented,
    'DIGEST_NONCE_BACKEND': NotImplemented,
    'DIGEST_ACCOUNT_BACKEND': NotImplemented}

class SettingsMixin(object):
    def __init__(self, *args, **kwargs):
        # Bind self.TestCase to the actual TestCase superclass
        self.superclass = None
        for cls in self.__class__.__bases__:
            if issubclass(cls, (TestCase, TransactionTestCase)):
                self.superclass = cls
        if not self.superclass:
            raise SyntaxError('SettingsMixin must be combined with a TestCase')
        super(SettingsMixin, self).__init__(*args, **kwargs)

    def __call__(self, result=None):
        with patch(settings, **DUMMY_SETTINGS):
            return self.superclass.__call__(self, result)

class UtilsTest(TestCase):
    def test_get_setting(self):
        with patch(settings,
                   A_PRESENT_SETTING='hello',
                   AN_ABSENT_SETTING=NotImplemented,
                   A_FALSE_SETTING=False):
            self.assertEqual('hello', get_setting('A_PRESENT_SETTING', 'blah'))
            self.assertEqual('blah', get_setting('AN_ABSENT_SETTING', 'blah'))
            self.assertEqual(False, get_setting('A_FALSE_SETTING', 'blah'))

    def test_get_backend(self):
        with patch(settings,
                   A_PRESENT_BACKEND_SETTING='django_digest.tests.DummyBackendClass',
                   AN_ABSENT_BACKEND_SETTING=NotImplemented):
            self.assertEqual(
                DummyBackendClass,
                type(get_backend('A_PRESENT_BACKEND_SETTING',
                                 'django_digest.tests.OtherDummyBackendClass')))

            self.assertEqual(
                OtherDummyBackendClass,
                type(get_backend('AN_ABSENT_BACKEND_SETTING',
                                 'django_digest.tests.OtherDummyBackendClass')))


class MockRequestMixin(object):
    def setUp(self):
        self.mocker = Mocker()
    
    def create_mock_request(self, username='dummy-username', realm=None,
                            method='GET', uri='/dummy/uri', nonce=None, request_digest=None,
                            algorithm=None, opaque='dummy-opaque', qop='auth', nonce_count=1,
                            client_nonce=None, password='password', request_path=None):
        if not realm:
            realm = get_setting('DIGEST_REALM', DEFAULT_REALM)
        if not nonce:
            nonce=python_digest.calculate_nonce(time.time(), secret=settings.SECRET_KEY)
        if not request_path:
            request_path = uri
        header = python_digest.build_authorization_request(
            username=username, realm=realm, method=method, uri=uri, nonce=nonce, opaque=opaque,
            nonce_count=nonce_count, password=password, request_digest=request_digest,
            client_nonce=client_nonce)

        request = self.create_mock_request_for_header(header)

        expect(request.method).result(method)
        expect(request.path).result(request_path)

        return request

    def create_mock_request_for_header(self, header):
        request = self.mocker.mock(HttpRequest, count=False)

        # bug in mocker: https://bugs.launchpad.net/mocker/+bug/179072
        try:
            'HTTP_AUTHORIZATION' in request.META
        except:
            pass
        self.mocker.result(not header == None)

        if header:
            expect(request.META['HTTP_AUTHORIZATION']).count(0, None).result(header)

        return request


class DjangoDigestTests(SettingsMixin, MockRequestMixin, TestCase):
    def test_build_challenge_response(self):
        response = HttpDigestAuthenticator().build_challenge_response()
        self.assertEqual(401, response.status_code)
        self.assertEqual('digest ', response["WWW-Authenticate"][:7].lower())
        parts = parse_parts(response["WWW-Authenticate"][7:])
        self.assertEqual(DEFAULT_REALM, parts['realm'])
        with patch(settings, DIGEST_REALM='CUSTOM'):
            response = HttpDigestAuthenticator().build_challenge_response()
            parts = parse_parts(response["WWW-Authenticate"][7:])
            self.assertEqual('CUSTOM', parts['realm'])

    def test_decorator_authenticated_with_parens(self):
        response = self.mocker.mock(count=False)
        expect(response.status_code).result(200)
        
        @httpdigest()
        def test_view(request):
            return response

        testuser = User.objects.create_user(username='testuser',
                                            email='user@example.com',
                                            password='pass')
        request = self.create_mock_request(username=testuser.username,
                                           password='pass')
        # expect the following assignment:
        request.user = testuser

        with self.mocker:
            self.assertEqual(response, test_view(request))
        
    def test_decorator_authenticated_without_parens(self):
        response = self.mocker.mock(count=False)
        expect(response.status_code).result(200)

        @httpdigest
        def test_view(request):
            return response

        testuser = User.objects.create_user(username='testuser',
                                            email='user@example.com',
                                            password='pass')
        request = self.create_mock_request(username=testuser.username,
                                           password='pass')
        request.user = testuser

        with self.mocker:
            self.assertEqual(response, test_view(request))
        
    def test_decorator_authenticated_with_full_uri(self):
        response = self.mocker.mock(count=False)
        expect(response.status_code).result(200)
        
        @httpdigest
        def test_view(request):
            return response

        testuser = User.objects.create_user('testuser', 'user@example.com', 'pass')
        request = self.create_mock_request(username=testuser.username, password='pass',
                                           uri='http://server:8000/some/path?q=v',
                                           request_path='/some/path')
        request.user = testuser

        with self.mocker:
            self.assertEqual(response, test_view(request))
        
    def test_decorator_with_realm_mismatch(self):
        response = self.mocker.mock(count=False)

        @httpdigest
        def test_view(request):
            return response

        testuser = User.objects.create_user('testuser', 'user@example.com', 'pass')
        request = self.create_mock_request(username=testuser.username, password='pass',
                                           realm='BAD_REALM')

        with self.mocker:
            final_response = test_view(request)
            self.assertEqual(401, final_response.status_code)
            self.assertTrue('WWW-Authenticate' in final_response)
        
    def test_decorator_unauthenticated_and_custom_settings(self):
        response = self.mocker.mock(count=False)
        expect(response.status_code).result(200)

        @httpdigest(realm='MY_TEST_REALM')
        def test_view(request):
            return response

        testuser = User.objects.create_user('testuser', 'user@example.com', 'pass')
        request = self.create_mock_request_for_header(None)
        request.user = testuser

        with self.mocker:
            final_response = test_view(request)
            self.assertEqual(401, final_response.status_code)
            self.assertTrue('WWW-Authenticate' in final_response)
            self.assertTrue('MY_TEST_REALM' in final_response['WWW-Authenticate'])
        
    def test_decorator_with_pre_constructed_authenticator(self):
        response = self.mocker.mock(count=False)
        expect(response.status_code).result(200)

        @httpdigest(HttpDigestAuthenticator(realm='MY_TEST_REALM'))
        def test_view(request):
            return response

        testuser = User.objects.create_user('testuser', 'user@example.com', 'pass')
        request = self.create_mock_request_for_header(None)
        request.user = testuser

        with self.mocker:
            final_response = test_view(request)
            self.assertEqual(401, final_response.status_code)
            self.assertTrue('WWW-Authenticate' in final_response)
            self.assertTrue('MY_TEST_REALM' in final_response['WWW-Authenticate'])

    def test_disable_nonce_count_enforcement(self):
        with patch(settings, DIGEST_ENFORCE_NONCE_COUNT=False):
            testuser = User.objects.create_user(username='testuser', 
                                                email='user@example.com',
                                                password='pass')
    
            nonce=python_digest.calculate_nonce(time.time(),
                                                secret=settings.SECRET_KEY)
    
            first_request = self.create_mock_request(
                username=testuser.username, password='pass', nonce=nonce)

            first_request.user = testuser
            
            # same nonce, same nonce count, will succeed
            second_request = self.create_mock_request(
                username=testuser.username, password='pass', nonce=nonce)
            second_request.user = testuser

            with self.mocker:
                authenticator = HttpDigestAuthenticator()
                self.assertTrue(authenticator.authenticate(first_request))
                self.assertTrue(authenticator.authenticate(second_request))

class DigestAuthenticateTransactionTests(SettingsMixin, MockRequestMixin,
                                         TransactionTestCase):
    def test_authenticate_nonce(self):
        testuser = User.objects.create_user(
            username='testuser', email='user@example.com', password='pass')
        otheruser = User.objects.create_user(
            username='otheruser', email='otheruser@example.com', password='pass')

        nonce=python_digest.calculate_nonce(time.time(), secret=settings.SECRET_KEY)

        first_request = self.create_mock_request(username=testuser.username,
                                                 password='pass', nonce=nonce)
        first_request.user = testuser

        # same nonce, same nonce count, will fail
        second_request = self.create_mock_request(username=testuser.username,
                                                  password='pass', nonce=nonce)

        # same nonce, new nonce count, it works
        third_request = self.create_mock_request(username=testuser.username,
                                                 password='pass', nonce=nonce,
                                                 nonce_count=2)
        third_request.user = testuser

        authenticator = HttpDigestAuthenticator()
        with self.mocker:
            self.assertTrue(HttpDigestAuthenticator.contains_digest_credentials(
                first_request
            ))
            with transaction.commit_on_success():
                self.assertTrue(authenticator.authenticate(first_request))
                self.assertFalse(authenticator.authenticate(second_request))
                transaction.rollback()
                self.assertTrue(authenticator.authenticate(third_request))

class DigestAuthenticateTests(SettingsMixin, MockRequestMixin, TestCase):
    def test_authenticate_invalid(self):
        testuser = User.objects.create_user(username='testuser',
                                            email='user@example.com',
                                            password='pass')
        otheruser = User.objects.create_user(username='otheruser',
                                             email='otheruser@example.com',
                                             password='pass')

        nonce=python_digest.calculate_nonce(time.time(),
                                            secret=settings.SECRET_KEY)
        # an invalid request
        fourth_request = self.create_mock_request_for_header(
            'Digest blah blah blah')

        # an invalid request
        fifth_request = self.create_mock_request_for_header(None)

        # an invalid nonce
        sixth_request = self.create_mock_request(
            username=testuser.username, password='pass', nonce_count=1,
            nonce=python_digest.calculate_nonce(time.time(), secret='bad secret'))

        # an invalid request digest (wrong password)
        seventh_request = self.create_mock_request(
            username=testuser.username, password='wrong', nonce=nonce,
            nonce_count=3)

        # attack attempts / failures don't invalidate the session or increment nonce_cont
        eighth_request = self.create_mock_request(
            username=testuser.username, password='pass', nonce=nonce,
            nonce_count=3)
        eighth_request.user = testuser

        # mismatched URI
        ninth_request = self.create_mock_request(
            username=testuser.username, nonce=nonce, password='pass',
            nonce_count=4, request_path='/different/path')

        # stale nonce
        tenth_request = self.create_mock_request(
            username=testuser.username, password='pass', nonce_count=4,
            nonce=python_digest.calculate_nonce(
                time.time()-60000, secret=settings.SECRET_KEY))

        # once the nonce is used by one user, can't be reused by another
        eleventh_request = self.create_mock_request(
            username=otheruser.username, password='pass', nonce=nonce,
            nonce_count=4)

        authenticator = HttpDigestAuthenticator()
        with self.mocker:
            self.assertFalse(authenticator.authenticate(fourth_request))
            self.assertFalse(authenticator.authenticate(fifth_request))
            self.assertFalse(authenticator.authenticate(sixth_request))
            self.assertFalse(authenticator.authenticate(seventh_request))
            self.assertTrue(authenticator.authenticate(eighth_request))
            self.assertFalse(authenticator.authenticate(ninth_request))
            self.assertFalse(authenticator.authenticate(tenth_request))
            self.assertFalse(authenticator.authenticate(eleventh_request))

    def test_authenticate_missing(self):
        testuser = User.objects.create_user(username='testuser',
                                            email='user@example.com',
                                            password='pass')
        otheruser = User.objects.create_user(username='otheruser',
                                             email='otheruser@example.com',
                                             password='pass')

        nonce=python_digest.calculate_nonce(time.time(),
                                            secret=settings.SECRET_KEY)
        # if the partial digest is not in the DB, authentication fails
        twelfth_request = self.create_mock_request(username=testuser.username,
                                                   password='pass',
                                                   nonce_count=3)

        authenticator = HttpDigestAuthenticator()
        with self.mocker:
            PartialDigest.objects.all().delete()
            self.assertFalse(authenticator.authenticate(twelfth_request))

    def test_authenticate_basic(self):
        # a request for Basic auth
        thirteenth_request = self.create_mock_request_for_header(
            'Basic YmxhaDpibGFo')

        authenticator = HttpDigestAuthenticator()
        with self.mocker:
            self.assertFalse(authenticator.authenticate(thirteenth_request))

class DummyLoginFactory(object):
    confirmed_logins = []
    unconfirmed_logins = []

    def confirmed_logins_for_user(self, user):
        return self.confirmed_logins

    def unconfirmed_logins_for_user(self, user):
        return self.unconfirmed_logins

class ModelsTests(TestCase):
    def test_unconfirmed_partial_digests(self):
        with patch(settings,
                   DIGEST_LOGIN_FACTORY='django_digest.tests.DummyLoginFactory'):
            with patch(DummyLoginFactory, confirmed_logins=['user', 'wierdo'],
                       unconfirmed_logins=['email@example.com', 'other@example.com']):
                user = User.objects.create_user(username='TestUser',
                                                password='password',
                                                email='TestUser@Example.com')
            self.assertEqual(
                set(['user', 'wierdo']),
                set([pd.login
                     for pd in PartialDigest.objects.filter(confirmed=True)]))
            user_pd = PartialDigest.objects.get(login='user').partial_digest
    
            self.assertEqual(
                set(['email@example.com', 'other@example.com']),
                set([pd.login
                     for pd in PartialDigest.objects.filter(confirmed=False)]))
            email_pd = PartialDigest.objects.get(
                login='email@example.com').partial_digest
    
            with patch(DummyLoginFactory,
                       confirmed_logins=['user', 'email@example.com'],
                       unconfirmed_logins=['wierdo']):
                from django_digest.backend.db import review_partial_digests
                review_partial_digests(user)
    
            self.assertEqual(
                set(['user', 'email@example.com']),
                set([pd.login
                     for pd in PartialDigest.objects.filter(confirmed=True)]))
    
            self.assertEqual(
                set(['wierdo']),
                set([pd.login
                     for pd in PartialDigest.objects.filter(confirmed=False)]))
            self.assertEqual(user_pd,
                             PartialDigest.objects.get(login='user').partial_digest)
            self.assertEqual(
                email_pd,
                PartialDigest.objects.get(login='email@example.com').partial_digest)


    def test_partial_digest_creation_on_set_password(self):
        user = User.objects.create_user(username='TestUser', password='password',
                                        email='TestUser@Example.com')
        expected = ['testuser', 'TestUser', 'testuser@example.com']

        # In Django < 1.2 the local name on emails is lowercased by create_user, so
        # expectations will be different for Django >= 1.2
        if user.email == 'TestUser@example.com':
            expected += [user.email]
            
        self.assertEqual(
            set(expected),
            set([pd.login for pd in PartialDigest.objects.all()]))

    def test_partial_digest_update_after_email_case_change(self):
        user = User.objects.create(username='TestUser', email='TestUser@example.com')
        user.set_password('password')
        user.save()
        expected = ['testuser', 'TestUser', 'TestUser@example.com',
                    'testuser@example.com']

        self.assertEqual(
            set(expected),
            set([pd.login for pd in PartialDigest.objects.all()]))
        user.email = 'tESTuSER@example.com'
        user.save()
        from django.contrib.auth import authenticate
        self.assertEqual(user,
                         authenticate(username='TestUser', password='password'))
        self.assertEqual(
            set(['testuser', 'TestUser', 'testuser@example.com',
                 'tESTuSER@example.com']),
            set([pd.login for pd in PartialDigest.objects.all()]))

    def test_partial_digest_creation_on_login(self):
        user = User.objects.create_user(username='TestUser', password='password',
                                        email='testuser@example.com')
        PartialDigest.objects.filter(user=user).delete()
        self.assertEqual(0,PartialDigest.objects.count())
        from django.contrib.auth import authenticate
        self.assertEqual(user, authenticate(username='TestUser', password='password'))
        self.assertEqual(
            set(['testuser', 'TestUser', 'testuser@example.com']),
            set([pd.login for pd in PartialDigest.objects.all()]))
        pks = [pd.pk for pd in PartialDigest.objects.all()]
        self.assertEqual(user, authenticate(username='TestUser', password='password'))
        # Ensure that the partial digests were not regenerated
        self.assertEqual(set(pks), set([pd.pk for pd in PartialDigest.objects.all()]))

    def test_partial_digest_creation_on_login_after_email_case_change(self):
        user = User.objects.create_user(username='TestUser', password='password',
                                        email='testuser@example.com')
        PartialDigest.objects.filter(user=user).delete()
        self.assertEqual(0,PartialDigest.objects.count())
        from django.contrib.auth import authenticate
        self.assertEqual(user, authenticate(username='TestUser', password='password'))
        self.assertEqual(
            set(['testuser', 'TestUser', 'testuser@example.com']),
            set([pd.login for pd in PartialDigest.objects.all()]))
        pks = [pd.pk for pd in PartialDigest.objects.all()]
        self.assertEqual(user, authenticate(username='TestUser', password='password'))
        # Ensure that the partial digests were not regenerated
        self.assertEqual(set(pks), set([pd.pk for pd in PartialDigest.objects.all()]))

        # Ensure that a case-only change in the email address does cause a
        # regeneration on login
        user.email = 'testUser@Example.com'
        user.save()
        self.assertEqual(user, authenticate(username='TestUser', password='password'))
        self.assertEqual(
            set(['testuser', 'TestUser', 'testuser@example.com',
                 'testUser@Example.com']),
            set([pd.login for pd in PartialDigest.objects.all()]))



class MiddlewareTests(SettingsMixin, TestCase):
    def setUp(self):
        self.mocker = Mocker()

    def test_valid_login(self):
        authenticator = self.mocker.mock()
        request = self.mocker.mock()
        expect(authenticator.authenticate(request)).result(True)
        with self.mocker:
            self.assertEqual(
                None, HttpDigestMiddleware(authenticator=authenticator).process_request(request))

    def test_no_login_and_not_required(self):
        authenticator = self.mocker.mock()
        request = self.mocker.mock()
        expect(authenticator.authenticate(request)).result(False)
        expect(authenticator.contains_digest_credentials(request)).result(False)
        with self.mocker:
            self.assertEqual(
                None, HttpDigestMiddleware(authenticator=authenticator).process_request(request))

    def test_no_login_and_required(self):
        authenticator = self.mocker.mock(count=False)
        request = self.mocker.mock()
        response = self.mocker.mock()
        expect(authenticator.authenticate(request)).result(False)
        expect(authenticator.contains_digest_credentials(request)).result(False)
        expect(authenticator.build_challenge_response()).result(response)
        with self.mocker:
            self.assertEqual(
                response,
                HttpDigestMiddleware(authenticator=authenticator,
                                     require_authentication=True).process_request(request))

    def test_process_response_401(self):
        authenticator = self.mocker.mock(count=False)
        request = self.mocker.mock()
        response = self.mocker.mock(count=False)
        challenge_response = self.mocker.mock()
        expect(response.status_code).result(401)
        expect(authenticator.build_challenge_response()).result(challenge_response)
        with self.mocker:
            self.assertEqual(
                challenge_response,
                HttpDigestMiddleware(authenticator=authenticator).process_response(
                    request, response))

    def test_process_response_403(self):
        authenticator = self.mocker.mock(count=False)
        request = self.mocker.mock()
        response = self.mocker.mock(count=False)
        challenge_response = self.mocker.mock()
        expect(response.status_code).result(403)
        expect(authenticator.build_challenge_response()).result(challenge_response)
        with self.mocker:
            self.assertEqual(
                challenge_response,
                HttpDigestMiddleware(authenticator=authenticator).process_response(
                    request, response))
        
    def test_process_response_200(self):
        authenticator = self.mocker.mock(count=False)
        request = self.mocker.mock()
        response = self.mocker.mock(count=False)
        expect(response.status_code).result(200)
        with self.mocker:
            self.assertEqual(
                response,
                HttpDigestMiddleware(authenticator=authenticator).process_response(
                    request, response))
        
    def test_process_response_404(self):
        authenticator = self.mocker.mock(count=False)
        request = self.mocker.mock()
        response = self.mocker.mock(count=False)
        expect(response.status_code).result(404)
        with self.mocker:
            self.assertEqual(
                response,
                HttpDigestMiddleware(authenticator=authenticator).process_response(
                    request, response))

class DbBackendTests(TestCase):
    def test_filter_out_inactive_users(self):
        user1 = User.objects.create(username='user1', email='user1@example.com',
                                    is_active=False)
        self.assertTrue(AccountStorage().get_partial_digest(user1.username)
                        is None)
        self.assertTrue(AccountStorage().get_user(user1.username) is None)
        user1.set_password('password')
        user1.save()
        self.assertTrue(AccountStorage().get_partial_digest(user1.username)
                        is None)
        self.assertTrue(AccountStorage().get_user(user1.username) is None)
        user1.is_active = True
        user1.save()
        self.assertTrue(AccountStorage().get_partial_digest(user1.username)
                        is not None)
        self.assertTrue(AccountStorage().get_user(user1.username) is not None)

    def test_get_partial_digest(self):
        user1 = User.objects.create(username='user1', email='user1@example.com')
        user2 = User.objects.create_user(username='User2', email='user2@example.com',
                                         password='pass')
        self.assertEqual(None, AccountStorage().get_partial_digest(user1.username))
        username_pd = AccountStorage().get_partial_digest(user2.username) 
        lowercase_username_pd = AccountStorage().get_partial_digest(
            user2.username.lower()) 
        email_pd = AccountStorage().get_partial_digest(user2.email) 
        self.assertTrue(username_pd is not None)
        self.assertTrue(lowercase_username_pd is not None)
        self.assertTrue(email_pd is not None)
        self.assertFalse(username_pd == email_pd)
        self.assertFalse(lowercase_username_pd == username_pd)
        self.assertEqual(None, AccountStorage().get_partial_digest('user3'))

    def test_get_user(self):
        user1 = User.objects.create_user(username='User1', email='user1@example.com',
                                         password='pass')
        user2 = User.objects.create_user(username='user2', email='user2@example.com',
                                         password='pass')
        self.assertFalse(user1.username == user1.username.lower())
        self.assertEqual(user1, AccountStorage().get_user(user1.username))
        self.assertEqual(user1, AccountStorage().get_user(user1.username.lower()))
        self.assertEqual(user2, AccountStorage().get_user(user2.username))
        self.assertEqual(None, AccountStorage().get_user('user3'))



from datetime import datetime

from django.db import IntegrityError

from django_digest.models import PartialDigest
from django_digest.utils import get_default_db


class AccountStorage(object):
    GET_PARTIAL_DIGEST_QUERY = """
    SELECT django_digest_partialdigest.login, django_digest_partialdigest.partial_digest
      FROM django_digest_partialdigest
      INNER JOIN auth_user ON auth_user.id = django_digest_partialdigest.user_id
      WHERE django_digest_partialdigest.login = %s
        AND django_digest_partialdigest.confirmed
        AND auth_user.is_active
    """

    def __init__(self, db=None):
        self.db = db or get_default_db()

    def get_partial_digest(self, username):
        cursor = self.db.connection.cursor()
        cursor.execute(self.GET_PARTIAL_DIGEST_QUERY, [username])
        # In MySQL, string comparison is case-insensitive by default.
        # Therefore a second round of filtering is required.
        row = [(row[1]) for row in cursor.fetchall() if row[0] == username]
        self.db.commit()
        if not row:
            return None
        return row[0]

    def get_user(self, username):
        # In MySQL, string comparison is case-insensitive by default.
        # Therefore a second round of filtering is required.
        pds = [pd
               for pd in PartialDigest.objects.filter(login=username,
                                                      user__is_active=True)
               if pd.login == username]
        if len(pds) == 0:
            return None
        if len(pds) > 1:
            _l.warn("Multiple partial digests found for the login %r" % username)
            return None
        return pds[0].user


class NonceStorage(object):
    DELETE_OLDER_THAN_QUERY = """
    SELECT django_digest_usernonce.last_used_at FROM django_digest_usernonce
      WHERE django_digest_usernonce.user_id = %s
      ORDER BY django_digest_usernonce.last_used_at DESC LIMIT 1 OFFSET 31
    """

    DELETE_EXPIRED_NONCES_QUERY = """
    DELETE FROM django_digest_usernonce
      WHERE django_digest_usernonce.last_used_at < %s
    """

    UPDATE_EXISTING_NONCE_WITH_COUNT_QUERY = """
    UPDATE django_digest_usernonce SET count = %s, last_used_at = %s
      WHERE django_digest_usernonce.nonce = %s
        AND django_digest_usernonce.user_id = %s
        AND django_digest_usernonce.count < %s
    """

    UPDATE_EXISTING_NONCE_WITHOUT_COUNT_QUERY = """
    UPDATE django_digest_usernonce SET count = NULL, last_used_at = %s
      WHERE django_digest_usernonce.nonce = %s
        AND django_digest_usernonce.user_id = %s
    """

    INSERT_NONCE_QUERY = """
    INSERT INTO django_digest_usernonce (user_id, nonce, count, last_used_at)
      VALUES (%s, %s, %s, %s)
    """

    def __init__(self, db=None):
        self.db = db or get_default_db()

    def _expire_nonces_for_user(self, user):
        cursor = self.db.connection.cursor()
        cursor.execute(self.DELETE_OLDER_THAN_QUERY, [user.id])
        row = cursor.fetchone()
        self.db.commit()
        if not row:
            return
        delete_older_than = row[0]
        cursor.execute(self.DELETE_EXPIRED_NONCES_QUERY, [delete_older_than])
        self.db.commit()

    def update_existing_nonce(self, user, nonce, nonce_count):
        cursor = self.db.connection.cursor()
        if nonce_count == None:
            cursor.execute(
                self.UPDATE_EXISTING_NONCE_WITHOUT_COUNT_QUERY,
                [self.db.connection.ops.value_to_db_datetime(datetime.now()),
                 nonce, user.id]
            )
        else:
            cursor.execute(
                self.UPDATE_EXISTING_NONCE_WITH_COUNT_QUERY,
                [nonce_count,
                 self.db.connection.ops.value_to_db_datetime(datetime.now()),
                 nonce, user.id, nonce_count]
            )
        self.db.commit()
        # if no rows are updated, either the nonce isn't in the DB,
        # it's for a different user, or the count is bad
        return cursor.rowcount == 1

    def store_nonce(self, user, nonce, nonce_count):
        self._expire_nonces_for_user(user)
        cursor = self.db.connection.cursor()
        try:
            cursor.execute(
                self.INSERT_NONCE_QUERY,
                [user.id, nonce, nonce_count,
                 self.db.connection.ops.value_to_db_datetime(datetime.now())]
            )
            return True
        except IntegrityError:
            return False
        finally:
            self.db.commit()



from django_digest.testsettings import *
INSTALLED_APPS += ('south',)

DATABASE_ENGINE = 'mysql' 
DATABASE_STORAGE_ENGINE = 'InnoDB'
DATABASE_HOST = 'localhost'
DATABASE_PORT = 3306
DATABASE_NAME = 'django_digest'
DATABASE_USER = 'root'
DATABASE_PASSWORD = ''

try:
    from django_digest.developer_settings import *
except ImportError:
    pass




from south.db import db
from django.db import models
from django_digest.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Changing field 'PartialDigest.login'
        # (to signature: django.db.models.fields.CharField(max_length=75, db_index=True))
        db.alter_column('django_digest_partialdigest', 'login', orm['django_digest.partialdigest:login'])
        
    
    
    def backwards(self, orm):
        
        # Changing field 'PartialDigest.login'
        # (to signature: django.db.models.fields.CharField(max_length=75, null=True, db_index=True))
        db.alter_column('django_digest_partialdigest', 'login', orm['django_digest.partialdigest:login'])
        
    
    
    models = {
        'auth.group': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '80', 'unique': 'True'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'unique_together': "(('content_type', 'codename'),)"},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'db_index': 'True', 'max_length': '75', 'unique': 'True', 'null': 'True', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'max_length': '30', 'unique': 'True'})
        },
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'django_digest.partialdigest': {
            'confirmed': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'login': ('django.db.models.fields.CharField', [], {'max_length': '75', 'db_index': 'True'}),
            'partial_digest': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'django_digest.usernonce': {
            'count': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_used_at': ('django.db.models.fields.DateTimeField', [], {}),
            'nonce': ('django.db.models.fields.CharField', [], {'max_length': '100', 'unique': 'True', 'db_index': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        }
    }
    
    complete_apps = ['django_digest']
